// mfcappView.cpp : implementation of the CMfcappView class
//
//
//	This code copyright 1997 Chris Losinger, unless otherwise noted
//
//	This code contains examples of using my JpegFile class, how to
//	read and write 1,4,8 and 24-bit BMPs and how to read and write
//	GIF87a files.
//
//	I find that this code works well for my purposes. Feel free to 
//	use it in your own code, but I can't assume any responsibility
//	if this code fails to do what you expect.
//
//	If you find any problems with this code, feel free to contact 
//	me for help.
//
//	Note : parts of the GIF specification are copyrighted. I am not 
//	authorized to grant license to use the GIF code (or to even use it
//	myself). If you worry about such things, I suggest you find out
//	how to get permission to use the GIF algorthim in your code. 
//
//


#include "stdafx.h"
#include "mfcapp.h"

#include "mfcDoc.h"
#include "mfcView.h"
#include "math.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#include "JpegFile.h"
#include "GIFFile.h"
#include "BMPDlg.h"

/////////////////////////////////////////////////////////////////////////////
// CMfcappView

IMPLEMENT_DYNCREATE(CMfcappView, CView)

BEGIN_MESSAGE_MAP(CMfcappView, CView)
	//{{AFX_MSG_MAP(CMfcappView)
	ON_COMMAND(ID_FILE_OPEN, OnFileOpen)
	ON_COMMAND(ID_FILE_SAVE_AS, OnFileSaveAs)
	ON_COMMAND(ID_FILE_SAVEGRAYAS, OnFileSavegrayas)
	ON_COMMAND(ID_FILE_GETDIMENSIONSJPG, OnFileGetdimensionsjpg)
	ON_COMMAND(ID_FILE_SAVECOLORMAPPEDBMP, OnFileSavecolormappedbmp)
	//}}AFX_MSG_MAP
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CView::OnFilePrintPreview)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CMfcappView construction/destruction

CMfcappView::CMfcappView()
{
	m_buf=NULL;
	m_width=0;
	m_height=0;
	m_widthDW=0;
}

CMfcappView::~CMfcappView()
{
	if (m_buf!=NULL) {
		delete [] m_buf;
		m_buf=NULL;
	}
}

BOOL CMfcappView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return CView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CMfcappView drawing

void CMfcappView::OnDraw(CDC* pDC)
{
	CMfcappDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	DrawBMP();
}

/////////////////////////////////////////////////////////////////////////////
// CMfcappView printing

BOOL CMfcappView::OnPreparePrinting(CPrintInfo* pInfo)
{
	// default preparation
	return DoPreparePrinting(pInfo);
}

void CMfcappView::OnBeginPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add extra initialization before printing
}

void CMfcappView::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add cleanup after printing
}

/////////////////////////////////////////////////////////////////////////////
// CMfcappView diagnostics

#ifdef _DEBUG
void CMfcappView::AssertValid() const
{
	CView::AssertValid();
}

void CMfcappView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CMfcappDoc* CMfcappView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CMfcappDoc)));
	return (CMfcappDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CMfcappView message handlers

void CMfcappView::OnFileOpen() 
{
	CString fileName;
	CString filt="JPG File (*.JPG)|*.JPG|BMP (*.BMP)|*.BMP|GIF (*.GIF)|*.GIF|All files (*.*)|*.*||";
    
    // OPENFILENAME - so i can get to its Help page easily
	CFileDialog *fileDlg;
	fileDlg = new CFileDialog(TRUE,"*.JPG","*.JPG",NULL,filt,this);
	if (fileDlg!=NULL) {
	    fileDlg->m_ofn.Flags|=OFN_FILEMUSTEXIST;
	    fileDlg->m_ofn.lpstrTitle="File to load";

		if (fileDlg->DoModal()==IDOK) {

			fileName=fileDlg->GetPathName();

			AfxGetApp()->DoWaitCursor(1);

			if ((fileName.Right(4)==".JPG") ||
				(fileName.Right(4)==".jpg"))
				LoadJPG(fileName);

			if ((fileName.Right(4)==".GIF") ||
				(fileName.Right(4)==".gif"))
				LoadGIF(fileName);

			if ((fileName.Right(4)==".BMP") ||
				(fileName.Right(4)==".bmp")) {
				LoadBMP(fileName);

				BYTE *tmp;
				tmp = JpegFile::MakeDwordAlignedBuf(m_buf,
											 m_width,
											 m_height,
											 &m_widthDW);
				delete [] m_buf;
				m_buf=tmp;

			}

			AfxGetApp()->DoWaitCursor(-1);

		}            
		delete fileDlg;
	}
		Invalidate(TRUE);

}

////////////////////////////////////////////////////////////////////////////
//	read a JPG to our global buffer
//

void CMfcappView::LoadJPG(CString fileName)
{

	// read to buffer tmp
	BYTE *tmp;
	tmp=JpegFile::JpegFileToRGB(fileName, &m_width, &m_height);
	if (tmp==NULL) {
		return;
	}

	// m_buf is the global buffer
	if (m_buf!=NULL) {
		delete [] m_buf;
		m_buf=NULL;
	}

	// do this before DWORD-alignment!!!
	// this works on packed (not DWORD-align buffers)
	// sawp red and blue for display
	JpegFile::BGRFromRGB(tmp, m_width, m_height);

	// DWORD-align for display
	m_buf = JpegFile::MakeDwordAlignedBuf(tmp,
									 m_width,
									 m_height,
									 &m_widthDW);

	delete [] tmp;

	// vertical flip for display
	JpegFile::VertFlipBuf(m_buf, m_widthDW, m_height);

}


////////////////////////////////////////////////////////////////////////////
//	save as dialog handler
//


void CMfcappView::OnFileSaveAs() 
{
	CString fileName;
	CString filt="JPG File (*.JPG)|*.JPG|BMP (*.BMP)|*.BMP|All files (*.*)|*.*||";
    
    // OPENFILENAME - so i can get to its Help page easily
	CFileDialog *fileDlg;
	fileDlg = new CFileDialog(FALSE,"*.JPG","*.JPG",NULL,filt,this);
	if (fileDlg!=NULL) {
	    fileDlg->m_ofn.Flags|=OFN_FILEMUSTEXIST;
	    fileDlg->m_ofn.lpstrTitle="File to save as";

		if (fileDlg->DoModal()==IDOK) {
			
			fileName=fileDlg->GetPathName();

			AfxGetApp()->DoWaitCursor(1);

			if ((fileName.Right(4)==".JPG") ||
				(fileName.Right(4)==".jpg"))
				SaveJPG(fileName,TRUE);

			if ((fileName.Right(4)==".BMP") ||
				(fileName.Right(4)==".bmp"))
				SaveBMP(fileName);

			AfxGetApp()->DoWaitCursor(-1);
		}            
		delete fileDlg;
	}



}


////////////////////////////////////////////////////////////////////////////
//
//	save a JPG

void CMfcappView::SaveJPG(CString fileName, BOOL color)
{
	BYTE *tmp=NULL;

	// we DWORD-align the buffers for display, undo the DWORD alignment
	// for saving.
	tmp=JpegFile::RGBFromDWORDAligned(m_buf,
									m_width,
									m_widthDW,
									m_height);

	// we vertical flip for display. undo that.
	JpegFile::VertFlipBuf(tmp, m_width * 3, m_height);

	// we swap red and blue for display, undo that
	JpegFile::BGRFromRGB(tmp, m_width, m_height);

	if (tmp==NULL) {
		AfxMessageBox("~DWORD Memory Error");
		return;
	}

	// save RGB packed buffer to JPG
	BOOL ok=JpegFile::RGBToJpegFile(fileName, 
						tmp,
						m_width,
						m_height,
						color, 
						75);
	if (!ok) {
		AfxMessageBox("Write Error");
	}
	
	if (tmp!=NULL)
		delete [] tmp;
}

////////////////////////////////////////////////////////////////////////////
//
//	save a grayscaled images

void CMfcappView::OnFileSavegrayas() 
{
	CString fileName;
	CString filt="JPG File (*.JPG)|*.JPG|GIF File (*.GIF)|*.GIF|BMP (*.BMP)|*.BMP|All files (*.*)|*.*||";
    
    // OPENFILENAME - so i can get to its Help page easily
	CFileDialog *fileDlg;
	fileDlg = new CFileDialog(FALSE,"*.JPG","*.JPG",NULL,filt,this);
	if (fileDlg!=NULL) {
	    fileDlg->m_ofn.Flags|=OFN_FILEMUSTEXIST;
	    fileDlg->m_ofn.lpstrTitle="File to save as";

		if (fileDlg->DoModal()==IDOK) {
		
			fileName=fileDlg->GetPathName();

				AfxGetApp()->DoWaitCursor(1);

				if ((fileName.Right(4)==".JPG") ||
					(fileName.Right(4)==".jpg"))
					SaveJPG(fileName,FALSE);

				if ((fileName.Right(4)==".BMP") ||
					(fileName.Right(4)==".bmp"))
					SaveBMP(fileName);

				if ((fileName.Right(4)==".GIF") ||
					(fileName.Right(4)==".gif"))
					SaveGIF(fileName);

				AfxGetApp()->DoWaitCursor(-1);

		}            
		delete fileDlg;
	}



}

////////////////////////////////////////////////////////////////////////////
//
//	get JPG dimensions

void CMfcappView::OnFileGetdimensionsjpg() 
{
	CString fileName;
	CString filt="JPG File (*.JPG)|*.JPG|All files (*.*)|*.*||";
    
    // OPENFILENAME - so i can get to its Help page easily
	CFileDialog *fileDlg;
	fileDlg = new CFileDialog(TRUE,"*.JPG","*.JPG",NULL,filt,this);
	if (fileDlg!=NULL) {
	    fileDlg->m_ofn.Flags|=OFN_FILEMUSTEXIST;
	    fileDlg->m_ofn.lpstrTitle="File to examine";

		if (fileDlg->DoModal()==IDOK) {
			fileName=fileDlg->GetPathName();
		}            
		delete fileDlg;
	}

	UINT width, height;
	if (JpegFile::GetJPGDimensions(fileName,
								&width,
								&height)) {
		char buf[200];
		sprintf(buf,"%d %d",width,height);
		AfxMessageBox(buf);
	} else
		AfxMessageBox("JPEG Error");
								   
}

////////////////////////////////////////////////////////////////////////////
//	load a .BMP file - 1,4,8,24 bit
//

void CMfcappView::LoadBMP(CString fileName)
{
	BITMAP inBM;

	BYTE m1,m2;
    long filesize;
    short res1,res2;
    long pixoff;
    long bmisize;                    
    long compression;
    unsigned long sizeimage;
    long xscale, yscale;
    long colors;
    long impcol;
    long bytesread=0;
	
	FILE *fp;
	
	fp=fopen(fileName,"rb");
	if (fp==NULL) {
		CString msg;                    
		msg="Can't open file for reading :\n"+fileName;
		AfxMessageBox(msg,MB_OK);
		return ;
	} else {
	    long rc;
		rc=fread((BYTE  *)&(m1),1,1,fp); bytesread+=1;
		if (rc==-1) AfxMessageBox("Read Error!");
		rc=fread((BYTE  *)&(m2),1,1,fp); bytesread+=1;
		if (rc==-1) AfxMessageBox("Read Error!");
		if ((m1!='B') || (m2!='M')) {
			AfxMessageBox("Not a valid BMP File");
			fclose(fp);
			return;
        }
        
		rc=fread((long  *)&(filesize),4,1,fp); bytesread+=4;
		if (rc==-1) AfxMessageBox("Read Error!");		
		rc=fread((int  *)&(res1),2,1,fp); bytesread+=2;
		if (rc==-1) AfxMessageBox("Read Error!");		
		rc=fread((int  *)&(res2),2,1,fp); bytesread+=2;
		if (rc==-1) AfxMessageBox("Read Error!");		
		rc=fread((long  *)&(pixoff),4,1,fp); bytesread+=4;
		if (rc==-1) AfxMessageBox("Read Error!");		
		rc=fread((long  *)&(bmisize),4,1,fp); bytesread+=4;
		if (rc==-1) AfxMessageBox("Read Error!");		

		rc=fread((long  *)&(inBM.bmWidth),4,1,fp);	 bytesread+=4;
		if (rc==-1) AfxMessageBox("Read Error!");		
		rc=fread((long  *)&(inBM.bmHeight),4,1,fp); bytesread+=4;
		if (rc==-1) AfxMessageBox("Read Error!");		
		rc=fread((int  *)&(inBM.bmPlanes),2,1,fp); bytesread+=2;
		if (rc==-1) AfxMessageBox("Read Error!");		
		rc=fread((int  *)&(inBM.bmBitsPixel),2,1,fp); bytesread+=2;
		if (rc==-1) AfxMessageBox("Read Error!");		
		rc=fread((long  *)&(compression),4,1,fp); bytesread+=4;
		if (rc==-1) AfxMessageBox("Read Error!");		
		rc=fread((long  *)&(sizeimage),4,1,fp); bytesread+=4;
		if (rc==-1) AfxMessageBox("Read Error!");		
		rc=fread((long  *)&(xscale),4,1,fp); bytesread+=4;
		if (rc==-1) AfxMessageBox("Read Error!");		
		rc=fread((long  *)&(yscale),4,1,fp); bytesread+=4;
		if (rc==-1) AfxMessageBox("Read Error!");		
		rc=fread((long  *)&(colors),4,1,fp); bytesread+=4;
		if (rc==-1) AfxMessageBox("Read Error!");		
		rc=fread((long  *)&(impcol),4,1,fp); bytesread+=4;
		if (rc==-1) AfxMessageBox("Read Error!");		
		
	    if (compression!=BI_RGB) {
	    	AfxMessageBox("This is a compressed file\nGiving up",MB_OK);
	    	fclose(fp);
	    	return;
	    }

		if (colors == 0) {
			colors = 1 << inBM.bmBitsPixel;
		}


		// read colormap
		RGBQUAD *colormap = NULL;

		switch (inBM.bmBitsPixel) {
		case 24:
			break;
			// read pallete 
		case 1:
		case 4:
		case 8:
			colormap = new RGBQUAD[colors];
			if (colormap==NULL) {
				fclose(fp);
				AfxMessageBox("Out of memory");
				return;
			}

			int i;
			for (i=0;i<colors;i++) {
				BYTE r,g,b, dummy;

				rc=fread((BYTE *)&(b),1,1,fp);
				bytesread++;
				if (rc==-1) AfxMessageBox("Read Error!");		

				rc=fread((BYTE  *)&(g),1,1,fp); 
				bytesread++;
				if (rc==-1) AfxMessageBox("Read Error!");		

				rc=fread((BYTE  *)&(r),1,1,fp); 
				bytesread++;
				if (rc==-1) AfxMessageBox("Read Error!");		

				rc=fread((BYTE  *)&(dummy),1,1,fp); 
				bytesread++;
				if (rc==-1) AfxMessageBox("Read Error!");		

				colormap[i].rgbRed=r;
				colormap[i].rgbGreen=g;
				colormap[i].rgbBlue=b;
			}
			break;
		}


		if (bytesread>pixoff) {
			fclose(fp);
			AfxMessageBox("Corrupt palette");
			delete [] colormap;
			return;
		}

		while (bytesread<pixoff) {
			char dummy;
			fread(&dummy,1,1,fp);
			bytesread++;
		}

		int w=inBM.bmWidth;
		int h=inBM.bmHeight;

		// globals for display
		m_width=w;
		m_height=h;

		long row_size = w * 3;

		long bufsize = (long)w * 3 * (long)h;

		// clear old image buffer
		if (m_buf!=NULL) {
			delete [] m_buf;
			m_buf=NULL;
		}
		
		// alloc our buffer
		m_buf=(BYTE *) new BYTE [bufsize];
		if (m_buf==NULL) {
			AfxMessageBox("Memory alloc Failed",MB_OK);
		} else {

			long row=0;
			long rowidx=0;

			for (row=0;row<inBM.bmHeight;row++) {

				// which row are we working on?
				rowidx=(long unsigned)row*row_size;						      

				if (inBM.bmBitsPixel==24) {
					
					// read a whole row
					fread((void  *)(m_buf+rowidx),row_size,1,fp);
					bytesread+=row_size;
					
					// read DWORD padding
					while ((bytesread-pixoff)&3) {
						char dummy;
						fread(&dummy,1,1,fp);
						bytesread++;
					}
				
				} else {	// 1, 4, or 8 bit image
					// pixels are packed as 1 , 4 or 8 bit vals. need to unpack them

					int bit_count = 0;
					UINT mask = (1 << inBM.bmBitsPixel) - 1;

					BYTE inbyte=0;

					for (int col=0;col<w;col++) {
						
						int pix=0;

						// if we need another byte
						if (bit_count <= 0) {
							bit_count = 8;
							fread(&inbyte,1,1,fp);
							bytesread++;
						}

						// keep track of where we are in the bytes
						bit_count -= inBM.bmBitsPixel;
						pix = ( inbyte >> bit_count) & mask;

						// lookup the color from the colormap - stuff it in our buffer
						*(m_buf + rowidx + col * 3 + 0) = colormap[pix].rgbBlue;
						*(m_buf + rowidx + col * 3 + 1) = colormap[pix].rgbGreen;
						*(m_buf + rowidx + col * 3 + 2) = colormap[pix].rgbRed;
					}

					// read DWORD padding
					while ((bytesread-pixoff)&3) {
						char dummy;
						fread(&dummy,1,1,fp);
						bytesread++;
					}
				}
			}
		
		}

		if (colormap) {
			delete [] colormap;
		}

		fclose(fp);

    }                 
}

////////////////////////////////////////////////////////////////////////////
//	write a 24-bit BMP file
//

void CMfcappView::SaveBMP(CString fileName)
{
	short res1=0;
    short res2=0;
    long pixoff=54;
    long compression=0;
    long cmpsize=0;
    long colors=0;
    long impcol=0;
	char m1='B';
	char m2='M';
	long bmfsize=sizeof(BITMAPFILEHEADER) + sizeof(BITMAPINFOHEADER) +
  							m_widthDW * m_height;	
	long byteswritten=0;

	BITMAPINFOHEADER header;
  	header.biSize=40; 						// header size
	header.biWidth=m_width;
	header.biHeight=m_height;
	header.biPlanes=1;
	header.biBitCount=24;					// RGB encoded, 24 bit
	header.biCompression=BI_RGB;			// no compression
	header.biSizeImage=0;
	header.biXPelsPerMeter=0;
	header.biYPelsPerMeter=0;
	header.biClrUsed=0;
	header.biClrImportant=0;

	FILE *fp;	
	fp=fopen(fileName,"wb");
	if (fp==NULL) {
		AfxMessageBox("Can't open file for writing",MB_OK);
		return;
	}

	// should probably check for write errors here...
	
	fwrite((BYTE  *)&(m1),1,1,fp); byteswritten+=1;
	fwrite((BYTE  *)&(m2),1,1,fp); byteswritten+=1;
	fwrite((long  *)&(bmfsize),4,1,fp);	byteswritten+=4;
	fwrite((int  *)&(res1),2,1,fp); byteswritten+=2;
	fwrite((int  *)&(res2),2,1,fp); byteswritten+=2;
	fwrite((long  *)&(pixoff),4,1,fp); byteswritten+=4;
	fwrite((long  *)&(header.biSize),4,1,fp); byteswritten+=4;

	fwrite((long  *)&(header.biWidth),4,1,fp); byteswritten+=4;
	fwrite((long  *)&(header.biHeight),4,1,fp); byteswritten+=4;
	fwrite((int  *)&(header.biPlanes),2,1,fp); byteswritten+=2;
	fwrite((int  *)&(header.biBitCount),2,1,fp); byteswritten+=2;

	fwrite((long  *)&(compression),4,1,fp); byteswritten+=4;
	fwrite((long  *)&(cmpsize),4,1,fp); byteswritten+=4;
	fwrite((long  *)&(header.biXPelsPerMeter),4,1,fp); byteswritten+=4;
	fwrite((long  *)&(header.biYPelsPerMeter),4,1,fp); byteswritten+=4;
	fwrite((long  *)&(colors),4,1,fp); byteswritten+=4;
	fwrite((long  *)&(impcol),4,1,fp); byteswritten+=4;
    
	long row=0;
	long rowidx;
	long row_size;
	row_size=header.biWidth*3;
    long rc;
	for (row=0;row<header.biHeight;row++) {
		rowidx=(long unsigned)row*row_size;						      

		rc=fwrite((void  *)(m_buf+rowidx),row_size,1,fp);
		if (rc==-1) {
			AfxMessageBox("fwrite error\nGiving up",MB_OK);
			break;
		}
	}
           
	fclose(fp);
}


////////////////////////////////////////////////////////////////////////////
//
//	shoot the global image to the screen

void CMfcappView::DrawBMP()
{
	BYTE *tmp = m_buf;
	if (tmp==NULL)
		return;

	CDC *theDC = GetDC();

	if (theDC!=NULL) {

		CRect clientRect;
		GetClientRect(clientRect);

		// Center It
		UINT left = (clientRect.Width() - m_width) / 2;
		UINT top = (clientRect.Height() - m_height) / 2;

		BITMAPINFOHEADER bmiHeader;
		bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
		bmiHeader.biWidth = m_width;
		bmiHeader.biHeight = m_height;
		bmiHeader.biPlanes = 1;
		bmiHeader.biBitCount = 24;
		bmiHeader.biCompression = BI_RGB;
		bmiHeader.biSizeImage = 0;
		bmiHeader.biXPelsPerMeter = 0;
		bmiHeader.biYPelsPerMeter = 0;
		bmiHeader.biClrUsed = 0;
		bmiHeader.biClrImportant = 0;


		// now blast it to the CDC passed in.
		// lines returns the number of lines actually displayed
		int lines = StretchDIBits(theDC->m_hDC,
					left, top,
					bmiHeader.biWidth,
					bmiHeader.biHeight,
					0,0,
					bmiHeader.biWidth,
					bmiHeader.biHeight,
					tmp,
					(LPBITMAPINFO)&bmiHeader,
					DIB_RGB_COLORS,
					SRCCOPY);
	
		ReleaseDC(theDC);
	}
}

////////////////////////////////////////////////////////////////////////////
//
//	read a GIF to our global buffer

void CMfcappView::LoadGIF(CString filename)
{
	GIFFile theGifThing;
	
	// read the GIF to a packed buffer of RGB bytes
	BYTE *tmp;
	tmp=theGifThing.GIFReadFileToRGB(filename, 
						&m_width, 
						&m_height);
	if (tmp==NULL) {
		AfxMessageBox(theGifThing.m_GIFErrorText);
		return;
	}

	if (m_buf!=NULL) {
		delete [] m_buf;
		m_buf=NULL;
	}

	// do this before DWORD-alignment!!!
	// swap red and blue for display
	JpegFile::BGRFromRGB(tmp, m_width, m_height);

	// now DWORD-align for display
	m_buf = JpegFile::MakeDwordAlignedBuf(tmp,
									 m_width,
									 m_height,
									 &m_widthDW);


	delete [] tmp;

	// flip for display
	JpegFile::VertFlipBuf(m_buf, m_widthDW, m_height);
}

////////////////////////////////////////////////////////////////////////////
//
//	save a 256 color GIF from our 24-bit global buffer

void CMfcappView::SaveGIF(CString filename)
{
	BYTE *tmp=NULL;

	// convert from DIB format (DWORD aligned, vertically flipped, red and blue swapped)
	tmp=JpegFile::RGBFromDWORDAligned(m_buf,
									m_width,
									m_widthDW,
									m_height);

	if (tmp==NULL) {
		AfxMessageBox("~DWORD Memory Error");
		return;
	}

	// convert from DIB
	JpegFile::VertFlipBuf(tmp, m_width * 3, m_height);

	// swap red and blue
	JpegFile::BGRFromRGB(tmp, m_width, m_height);

	// GIFs want 8-bit images, not 24-bit
	BYTE * tmp2 = Make256ColorGrayScaleFromRGB(tmp, m_width, m_height);
	
	delete [] tmp;

	if (tmp2==NULL) {
		AfxMessageBox("Failed to make 256 color image");
		return;
	}

	// simple gray scale palette - i'm not going to bother with color quantization
	int red[256], blue[256], green[256];
	for (int i=0;i<256;i++) {
		red[i]=i;
		green[i]=i;
		blue[i]=i;
	}

	GIFFile theGifThing;
	if (!theGifThing.GIFWriteFileFrom256Color(tmp2,
							filename,
							m_width, 
							m_height,
							0,		// background color
							red, green, blue)) {
		AfxMessageBox(theGifThing.m_GIFErrorText);
	}

	if (tmp2!=NULL)
		delete [] tmp2;

}

////////////////////////////////////////////////////////////////////////////////////
//
//	1,4,8 bit BMP stuff
//
//	if you have a color-mapped image and a color map...
//
//	the BMP saving code in SaveColorMappedBMP modified from Programming 
//	for Graphics Files in C and C++, by John Levine.

#define BMP_HEADERSIZE (3 * 2 + 4 * 12)
#define BMP_BYTESPERLINE (width, bits) ((((width) * (bits) + 31) / 32) * 4)
#define BMP_PIXELSIZE(width, height, bits) (((((width) * (bits) + 31) / 32) * 4) * height)

void CMfcappView::SaveColorMappedBMP(CString fileName, 
									 BYTE * colormappedbuffer,	// one BYTE per pixel colomapped image
									 int bitsperpixel,			// 1, 4, 8
									 int colors,				// number of colors (number of RGBQUADs)
									 RGBQUAD *colormap)			// array of RGBQUADs 
{
	int datasize, cmapsize, byteswritten, row, col;

	if (bitsperpixel == 24) {
		// the routines could be combined, but i don't feel like it
		AfxMessageBox("We don't do 24-bit files in here, sorry");
		return;
	} else
		cmapsize = colors * 4;

	datasize = BMP_PIXELSIZE(m_width, m_height, bitsperpixel);

	char m1 = 'B';
	char m2 = 'M';
	long filesize = BMP_HEADERSIZE + cmapsize + datasize;
	int res1, res2;
	res1 = res2 = 0;

	long pixeloffset = BMP_HEADERSIZE + cmapsize;

	int bmisize = 40;
	long cols = m_width;
	long rows = m_height;
	long planes = 1;
	long compression =0;
	long cmpsize = datasize;
	long xscale = 0;
	long yscale = 0;
	long impcolors = colors;

	FILE *fp;
	fp = fopen(fileName, "wb");
	if (fp==NULL) {
		AfxMessageBox("Can't Open");
		return;
	}

	fwrite(&m1, 1, 1, fp);
	fwrite(&m2, 1, 1, fp);
	fwrite(&filesize, 4 , 1, fp);
	fwrite(&res1, 2, 1, fp);
	fwrite(&res2, 2, 1, fp);
	fwrite(&pixeloffset, 4, 1, fp);
	fwrite(&bmisize, 4, 1, fp);
	fwrite(&cols, 4, 1, fp);
	fwrite(&rows, 4, 1, fp);
	fwrite(&planes, 2, 1, fp);
	fwrite(&bitsperpixel, 2, 1, fp);
	fwrite(&compression, 4, 1, fp);
	fwrite(&cmpsize, 4, 1, fp);
	fwrite(&xscale, 4, 1, fp);
	fwrite(&yscale, 4, 1, fp);
	fwrite(&colors, 4, 1, fp);
	fwrite(&impcolors, 4, 1, fp);

	if (cmapsize) {
		int i;
		for (i = 0; i< colors; i++) {
			putc(colormap[i].rgbBlue, fp);
			putc(colormap[i].rgbGreen, fp);
			putc(colormap[i].rgbRed, fp);
			putc(0, fp);	// dummy
		}
	}

	byteswritten = BMP_HEADERSIZE + cmapsize;

	for (row = m_height; --row >=0; ) {
		int pixbuf;
		int nbits = 0;

		for (col =0 ; col < (int)m_width; col++) {
			int offset = row * m_width + col;	// offset into our color-mapped RGB buffer
			BYTE pval = *(colormappedbuffer + offset);

			pixbuf = (pixbuf << bitsperpixel) | pval;

			nbits += bitsperpixel;

			if (nbits > 8) {
				AfxMessageBox("?????nBits > 8????");
				fclose(fp);
				return;
			}

			if (nbits == 8) {
				putc(pixbuf, fp);
				pixbuf=0;
				nbits=0;
				byteswritten++;
			}
		} // cols

		if (nbits > 0) {
			putc(pixbuf, fp);		// write partially filled byte
			byteswritten++;
		}

		// DWORD align
		while ((byteswritten -pixeloffset) & 3) {
			putc(0, fp);
			byteswritten++;
		}

	}	//rows

	if (byteswritten!=filesize) {
		AfxMessageBox("byteswritten != filesize");
	}

	fclose(fp);
}

////////////////////////////////////////////////////////////////////////////
// just use a luminance calculation to make a grayscale image
// it might be nice to use color quantization, but that's tough to implement :)

BYTE * CMfcappView::Make256ColorGrayScaleFromRGB(BYTE * inBuf,
												 UINT m_width, 
												 UINT m_height)
{
	BYTE *outBuf;
	outBuf = new BYTE[m_width * m_height];
	if (outBuf==NULL)
		return NULL;

	UINT col, row;
	long inOffset;
	long outOffset;
	for (row=0;row<m_height;row++) {
		for (col=0;col<m_width;col++) {
			outOffset = row * m_width + col;
			inOffset = row * m_width * 3 + col * 3;
	
			int r = *(inOffset + 0 + inBuf);
			int g = *(inOffset + 1 + inBuf);
			int b = *(inOffset + 2 + inBuf);

			int lum = (int)((.299 * (double)r) + (.587 * (double)g) + (.114 * (double)b));

			*(outBuf + outOffset) = lum;
		}
	}

	return outBuf;
}

////////////////////////////////////////////////////////////////////////////
//
//	save 1,4,8 bit BMP from our 24-bit global buffer

void CMfcappView::OnFileSavecolormappedbmp() 
{
	CBMPDlg theDlg;

	int bitsperpixel = 8;

	if (theDlg.DoModal()==IDOK) {
		bitsperpixel = theDlg.m_bits;
	} else return;

	CString fileName;
	CString filt="BMP (*.BMP)|*.BMP|All files (*.*)|*.*||";
    
    // OPENFILENAME - so i can get to its Help page easily
	CFileDialog *fileDlg;
	fileDlg = new CFileDialog(FALSE,"*.BMP","*.BMP",NULL,filt,this);
	if (fileDlg!=NULL) {
	    fileDlg->m_ofn.Flags|=OFN_FILEMUSTEXIST;
	    fileDlg->m_ofn.lpstrTitle="File to save as";

		if (fileDlg->DoModal()==IDOK) {
			
			fileName=fileDlg->GetPathName();

			AfxGetApp()->DoWaitCursor(1);

			RGBQUAD colormap[256];

			int colors = (int)pow(2,bitsperpixel);

			// based on bitsperpixel, create a colormapped image
			// this just does grayscale, sorry
			// i don't have the brains to write a good color quantization routine

			BYTE *colormappedbuffer = MakeColormapped(m_buf, 
													(UINT)m_width, 
													(UINT)m_height, 
													(UINT)m_widthDW, 
													(UINT)colors,
													colormap);

			if (colormappedbuffer!=NULL) {

				// write the MBP using our colormapped image (one byte per pixel),
				// number of bits, number of total colors and a colormap
				// pixels must be in the range 0...colors-1
				SaveColorMappedBMP(fileName,
									colormappedbuffer,
									bitsperpixel,
									colors,			// number of colors (number of RGBQUADs)
									colormap);		// array of RGBQUADs 
				

				// toss our buffer...
				delete [] colormappedbuffer;

			} else {
				AfxMessageBox("Failed to allocate space for RGB buffer");
			}

			AfxGetApp()->DoWaitCursor(-1);
		}            
		delete fileDlg;
	}
}

//
//	instead of creating a good palette for the colormapped images
//	this just graymaps them.
//

BYTE * CMfcappView::MakeColormapped(BYTE *inBuf,
								  UINT inWidth,
								  UINT inHeight,
								  UINT inWidthBytes,
								  UINT colors,
								  RGBQUAD* colormap)
{
	BYTE *tmp;

	// allocate a buffer to colormap
	tmp = (BYTE *)  new BYTE[inWidth * inHeight];
	if (tmp==NULL)
		return NULL;

	// force our image to use a stupid gray scale
	UINT color;
	for (color = 0;color < colors; color++) {
		colormap[color].rgbRed = color * 256 / colors;
		colormap[color].rgbGreen = color * 256 / colors;
		colormap[color].rgbBlue = color * 256 / colors;
	}


	UINT col, row;
	for (row =0; row < inHeight; row++) {

		for (col=0;col <inWidth; col++) {
			
			BYTE inRed, inBlue, inGreen;

			// src pixel
			long in_offset = row * inWidthBytes + col * 3;
			inRed = *(inBuf + in_offset + 0);
			inGreen = *(inBuf + in_offset + 1);
			inBlue = *(inBuf + in_offset + 2);

			// luminance
			int lum = (int)(.299 * (double)(inRed) + 
							.587 * (double)(inGreen) + 
							.114 * (double)(inBlue));

			// force luminance value into our range of colors
			lum = colors * lum / 256;

			// dest pixel
			long out_offset = row * inWidth + col;
			*(tmp+out_offset) = (BYTE)lum;
		}
	}

	// cause BMPs are upside down ?
	JpegFile::VertFlipBuf(tmp, m_width, m_height);

	return tmp;
}

